@@ -21,7 +21,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifndef _MSC_VER
+#if !defined _MSC_VER && !defined __MINGW32__
 /* unlink is in stdio.h in VC++ */
 #include <unistd.h> /* for unlink() */
 #else
@@ -33,20 +33,21 @@
 #include "encode.h"
 #include "file.h"
 
-static int usage(const char *message, ...);
+static int short_usage(const char *message, ...);
+static int long_usage(const char *message, ...);
 static int encode_file(const char *infilename, const char *forced_outfilename, FLAC__bool is_last_file);
 static int decode_file(const char *infilename, const char *forced_outfilename);
 
-FLAC__bool verify = false, verbose = true, lax = false, test_only = false, analyze = false;
+FLAC__bool verify = false, verbose = true, lax = false, test_only = false, analyze = false, use_ogg = false;
 FLAC__bool do_mid_side = true, loose_mid_side = false, do_exhaustive_model_search = false, do_escape_coding = false, do_qlp_coeff_prec_search = false;
-FLAC__bool force_to_stdout = false, delete_input = false, sector_align = false;
+FLAC__bool force_to_stdout = false, force_raw_format = false, delete_input = false, sector_align = false;
 const char *cmdline_forced_outfilename = 0, *output_prefix = 0;
 analysis_options aopts = { false, false };
 unsigned padding = 0;
 unsigned max_lpc_order = 8;
 unsigned qlp_coeff_precision = 0;
 FLAC__uint64 skip = 0;
-int format_is_wave = -1, format_is_big_endian = -1, format_is_unsigned_samples = false;
+int format_is_big_endian = -1, format_is_unsigned_samples = false;
 int format_channels = -1, format_bps = -1, format_sample_rate = -1;
 int blocksize = -1, min_residual_partition_order = -1, max_residual_partition_order = -1, rice_parameter_search_dist = -1;
 char requested_seek_points[50000]; /* @@@ bad MAGIC NUMBER */
@@ -55,19 +56,23 @@ FLAC__int32 align_reservoir_0[588], align_reservoir_1[588]; /* for carrying over
 FLAC__int32 *align_reservoir[2] = { align_reservoir_0, align_reservoir_1 };
 unsigned align_reservoir_samples = 0; /* 0 .. 587 */
 
+static const char *flac_suffix = ".flac", *ogg_suffix = ".ogg";
+
 int main(int argc, char *argv[])
 {
 	int i, retval = 0;
 	FLAC__bool mode_decode = false;
 
 	if(argc <= 1)
-		return usage(0);
+		return short_usage(0);
 
 	/* get the options */
 	for(i = 1; i < argc; i++) {
 		if(argv[i][0] != '-' || argv[i][1] == 0)
 			break;
-		if(0 == strcmp(argv[i], "-d"))
+		if(0 == strcmp(argv[i], "-H") || 0 == strcmp(argv[i], "--help"))
+			return long_usage(0);
+		else if(0 == strcmp(argv[i], "-d"))
 			mode_decode = true;
 		else if(0 == strcmp(argv[i], "-a")) {
 			mode_decode = true;
@@ -110,6 +115,12 @@ int main(int argc, char *argv[])
 			lax = true;
 		else if(0 == strcmp(argv[i], "--lax-"))
 			lax = false;
+#ifdef FLAC__HAS_OGG
+		else if(0 == strcmp(argv[i], "--ogg"))
+			use_ogg = true;
+		else if(0 == strcmp(argv[i], "--ogg-"))
+			use_ogg = false;
+#endif
 		else if(0 == strcmp(argv[i], "-b"))
 			blocksize = atoi(argv[++i]);
 		else if(0 == strcmp(argv[i], "-e"))
@@ -172,9 +183,7 @@ int main(int argc, char *argv[])
 		else if(0 == strcmp(argv[i], "-fu"))
 			format_is_unsigned_samples = true;
 		else if(0 == strcmp(argv[i], "-fr"))
-			format_is_wave = false;
-		else if(0 == strcmp(argv[i], "-fw"))
-			format_is_wave = true;
+			force_raw_format = true;
 		else if(0 == strcmp(argv[i], "--a-rgp"))
 			aopts.do_residual_gnuplot = true;
 		else if(0 == strcmp(argv[i], "--a-rgp-"))
@@ -277,7 +286,7 @@ int main(int argc, char *argv[])
 			rice_parameter_search_dist = 0;
 			max_lpc_order = 12;
 		}
-		else if(0 == strcmp(argv[i], "-9")) {
+		else if(0 == strcmp(argv[i], "--super-secret-impractical-compression-level")) {
 			do_exhaustive_model_search = true;
 			do_escape_coding = true;
 			do_mid_side = true;
@@ -289,10 +298,10 @@ int main(int argc, char *argv[])
 			max_lpc_order = 32;
 		}
 		else if(isdigit((int)(argv[i][1]))) {
-			return usage("ERROR: compression level '%s' is still reserved\n", argv[i]);
+			return long_usage("ERROR: compression level '%s' is reserved\n", argv[i]);
 		}
 		else {
-			return usage("ERROR: invalid option '%s'\n", argv[i]);
+			return long_usage("ERROR: invalid option '%s'\n", argv[i]);
 		}
 	}
 
@@ -322,7 +331,7 @@ int main(int argc, char *argv[])
 	else {
 		if(test_only) {
 			if(skip > 0)
-				return usage("ERROR: --skip is not allowed in test mode\n");
+				return long_usage("ERROR: --skip is not allowed in test mode\n");
 		}
 	}
 
@@ -330,38 +339,38 @@ int main(int argc, char *argv[])
 
 	if(format_channels >= 0) {
 		if(format_channels == 0 || (unsigned)format_channels > FLAC__MAX_CHANNELS)
-			return usage("ERROR: invalid number of channels '%u', must be > 0 and <= %u\n", format_channels, FLAC__MAX_CHANNELS);
+			return long_usage("ERROR: invalid number of channels '%u', must be > 0 and <= %u\n", format_channels, FLAC__MAX_CHANNELS);
 	}
 	if(format_bps >= 0) {
 		if(format_bps != 8 && format_bps != 16 && format_bps != 24)
-			return usage("ERROR: invalid bits per sample '%u' (must be 8/16/24)\n", format_bps);
+			return long_usage("ERROR: invalid bits per sample '%u' (must be 8/16/24)\n", format_bps);
 	}
 	if(format_sample_rate >= 0) {
 		if(format_sample_rate == 0 || (unsigned)format_sample_rate > FLAC__MAX_SAMPLE_RATE)
-			return usage("ERROR: invalid sample rate '%u', must be > 0 and <= %u\n", format_sample_rate, FLAC__MAX_SAMPLE_RATE);
+			return long_usage("ERROR: invalid sample rate '%u', must be > 0 and <= %u\n", format_sample_rate, FLAC__MAX_SAMPLE_RATE);
 	}
 	if(!mode_decode && ((unsigned)blocksize < FLAC__MIN_BLOCK_SIZE || (unsigned)blocksize > FLAC__MAX_BLOCK_SIZE)) {
-		return usage("ERROR: invalid blocksize '%u', must be >= %u and <= %u\n", (unsigned)blocksize, FLAC__MIN_BLOCK_SIZE, FLAC__MAX_BLOCK_SIZE);
+		return long_usage("ERROR: invalid blocksize '%u', must be >= %u and <= %u\n", (unsigned)blocksize, FLAC__MIN_BLOCK_SIZE, FLAC__MAX_BLOCK_SIZE);
 	}
 	if(qlp_coeff_precision > 0 && qlp_coeff_precision < FLAC__MIN_QLP_COEFF_PRECISION) {
-		return usage("ERROR: invalid value for -q '%u', must be 0 or >= %u\n", qlp_coeff_precision, FLAC__MIN_QLP_COEFF_PRECISION);
+		return long_usage("ERROR: invalid value for -q '%u', must be 0 or >= %u\n", qlp_coeff_precision, FLAC__MIN_QLP_COEFF_PRECISION);
 	}
 
 	if(sector_align) {
 		if(mode_decode)
-			return usage("ERROR: --sector-align only allowed for encoding\n");
+			return long_usage("ERROR: --sector-align only allowed for encoding\n");
 		else if(skip > 0)
-			return usage("ERROR: --sector-align not allowed with --skip\n");
+			return long_usage("ERROR: --sector-align not allowed with --skip\n");
 		else if(format_channels >= 0 && format_channels != 2)
-			return usage("ERROR: --sector-align can only be done with stereo input\n");
+			return long_usage("ERROR: --sector-align can only be done with stereo input\n");
 		else if(format_sample_rate >= 0 && format_sample_rate != 2)
-			return usage("ERROR: --sector-align can only be done with sample rate of 44100\n");
+			return long_usage("ERROR: --sector-align can only be done with sample rate of 44100\n");
 	}
 	if(argc - i > 1 && cmdline_forced_outfilename) {
-		return usage("ERROR: -o cannot be used with multiple files\n");
+		return long_usage("ERROR: -o cannot be used with multiple files\n");
 	}
 	if(cmdline_forced_outfilename && output_prefix) {
-		return usage("ERROR: --output-prefix conflicts with -o\n");
+		return long_usage("ERROR: --output-prefix conflicts with -o\n");
 	}
 
 	if(verbose) {
@@ -371,8 +380,17 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "welcome to redistribute it under certain conditions.  Type `flac' for details.\n\n");
 
 		if(!mode_decode) {
-			fprintf(stderr, "options:%s%s%s -P %u -b %u%s -l %u%s%s%s -q %u -r %u,%u -R %u%s\n",
-				delete_input?" --delete-input-file":"", sector_align?" --sector-align":"", lax?" --lax":"",
+			fprintf(stderr,
+				"options:%s%s"
+#ifdef FLAC__HAS_OGG
+				"%s"
+#endif
+				"%s -P %u -b %u%s -l %u%s%s%s -q %u -r %u,%u -R %u%s\n",
+				delete_input?" --delete-input-file":"", sector_align?" --sector-align":"",
+#ifdef FLAC__HAS_OGG
+				use_ogg?" --ogg":"",
+#endif
+				lax?" --lax":"",
 				padding, (unsigned)blocksize, loose_mid_side?" -M":do_mid_side?" -m":"", max_lpc_order,
 				do_exhaustive_model_search?" -e":"", do_escape_coding?" -E":"", do_qlp_coeff_prec_search?" -p":"",
 				qlp_coeff_precision,
@@ -384,7 +402,6 @@ int main(int argc, char *argv[])
 
 	if(mode_decode) {
 		FLAC__bool first = true;
-		int save_format;
 
 		if(i == argc) {
 			retval = decode_file("-", 0);
@@ -395,16 +412,13 @@ int main(int argc, char *argv[])
 			for(retval = 0; i < argc && retval == 0; i++) {
 				if(0 == strcmp(argv[i], "-") && !first)
 					continue;
-				save_format = format_is_wave;
 				retval = decode_file(argv[i], 0);
-				format_is_wave = save_format;
 				first = false;
 			}
 		}
 	}
 	else { /* encode */
 		FLAC__bool first = true;
-		int save_format;
 
 		if(i == argc) {
 			retval = encode_file("-", 0, true);
@@ -415,9 +429,7 @@ int main(int argc, char *argv[])
 			for(retval = 0; i < argc && retval == 0; i++) {
 				if(0 == strcmp(argv[i], "-") && !first)
 					continue;
-				save_format = format_is_wave;
 				retval = encode_file(argv[i], 0, i == (argc-1));
-				format_is_wave = save_format;
 				first = false;
 			}
 		}
@@ -426,18 +438,8 @@ int main(int argc, char *argv[])
 	return retval;
 }
 
-int usage(const char *message, ...)
+static void usage_header()
 {
-	va_list args;
-
-	if(message) {
-		va_start(args, message);
-
-		(void) vfprintf(stderr, message, args);
-
-		va_end(args);
-
-	}
 	fprintf(stderr, "===============================================================================\n");
 	fprintf(stderr, "flac - Command-line FLAC encoder/decoder version %s\n", FLAC__VERSION_STRING);
 	fprintf(stderr, "Copyright (C) 2000,2001  Josh Coalson\n");
@@ -456,98 +458,151 @@ int usage(const char *message, ...)
 	fprintf(stderr, "along with this program; if not, write to the Free Software\n");
 	fprintf(stderr, "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.\n");
 	fprintf(stderr, "===============================================================================\n");
-	fprintf(stderr, "Usage:\n");
-	fprintf(stderr, "  flac [options] [infile [...]]\n");
+}
+
+int short_usage(const char *message, ...)
+{
+	va_list args;
+
+	if(message) {
+		va_start(args, message);
+
+		(void) vfprintf(stderr, message, args);
+
+		va_end(args);
+
+	}
+	usage_header();
 	fprintf(stderr, "\n");
-	fprintf(stderr, "For encoding:\n");
-	fprintf(stderr, "  the input file(s) may be a PCM RIFF WAVE file or raw samples\n");
-	fprintf(stderr, "  the output file(s) will be in FLAC format\n");
-	fprintf(stderr, "For decoding, the reverse is true\n");
+	fprintf(stderr, "This is the short help; for full help use 'flac --help'\n");
 	fprintf(stderr, "\n");
-	fprintf(stderr, "A single 'infile' may be - for stdin.  No 'infile' implies stdin.  Use of\n");
-	fprintf(stderr, "stdin implies -c (write to stdout).  Normally you should use:\n");
-	fprintf(stderr, "   flac [options] -o outfilename  or  flac -d [options] -o outfilename\n");
-	fprintf(stderr, "instead of:\n");
-	fprintf(stderr, "   flac [options] > outfilename   or  flac -d [options] > outfilename\n");
-	fprintf(stderr, "since the former allows flac to seek backwards to write the STREAMINFO or\n");
-	fprintf(stderr, "RIFF WAVE header contents when necessary.\n");
+	fprintf(stderr, "To encode:\n");
+	fprintf(stderr, "  flac [-#] [infile [...]]\n");
 	fprintf(stderr, "\n");
-	fprintf(stderr, "If the unencoded filename ends with '.wav' or -fw is used, it's assumed to be\n");
-	fprintf(stderr, "RIFF WAVE.  Otherwise, flac will check for the presence of a RIFF header.  If\n");
-	fprintf(stderr, "any infile is raw you must specify the format options {-fb|fl} -fc -fp and -fs,\n");
-	fprintf(stderr, "which will apply to all raw files.  You can force a .wav file to be treated as\n");
-	fprintf(stderr, "a raw file using -fr.\n");
+	fprintf(stderr, "  -# is -0 (fastest compression) to -8 (highest compression); -5 is the default\n");
 	fprintf(stderr, "\n");
-	fprintf(stderr, "generic options:\n");
-	fprintf(stderr, "  -d : decode (default behavior is encode)\n");
-	fprintf(stderr, "  -t : test (same as -d except no decoded file is written)\n");
-	fprintf(stderr, "  -a : analyze (same as -d except an analysis file is written)\n");
-	fprintf(stderr, "  -c : write output to stdout\n");
-	fprintf(stderr, "  -s : silent (do not write runtime encode/decode statistics)\n");
-	fprintf(stderr, "  -o filename : force the output file name (usually flac just changes the\n");
-	fprintf(stderr, "                extension)\n");
-	fprintf(stderr, "  --delete-input-file : deletes the input file after a successful encode/decode\n");
-	fprintf(stderr, "  --skip samples : can be used both for encoding and decoding\n");
-	fprintf(stderr, "analyze options:\n");
-	fprintf(stderr, "  --a-rtext : include residual signal in text output\n");
-	fprintf(stderr, "  --a-rgp : generate gnuplot files of residual distribution of each subframe\n");
-	fprintf(stderr, "encoding options:\n");
-	fprintf(stderr, "  --lax : allow encoder to generate non-Subset files\n");
-	fprintf(stderr, "  --sector-align : align encoding of multiple files on sector boundaries\n");
-	fprintf(stderr, "  -S { # | X | #x } : include a point or points in a SEEKTABLE\n");
-	fprintf(stderr, "       #  : a specific sample number for a seek point\n");
-	fprintf(stderr, "       X  : a placeholder point (always goes at the end of the SEEKTABLE)\n");
-	fprintf(stderr, "       #x : # evenly spaced seekpoints, the first being at sample 0\n");
-	fprintf(stderr, "     You may use many -S options; the resulting SEEKTABLE will be the unique-\n");
-	fprintf(stderr, "           ified union of all such values.\n");
-	fprintf(stderr, "     With no -S options, flac defaults to '-S 100x'.  Use -S- for no SEEKTABLE.\n");
-	fprintf(stderr, "     Note: -S #x will not work if the encoder can't determine the input size\n");
-	fprintf(stderr, "           before starting.\n");
-	fprintf(stderr, "     Note: if you use -S # and # is >= samples in the input, there will be\n");
-	fprintf(stderr, "           either no seek point entered (if the input size is determinable\n");
-	fprintf(stderr, "           before encoding starts) or a placeholder point (if input size is not\n");
-	fprintf(stderr, "           determinable)\n");
-	fprintf(stderr, "  -P # : write a PADDING block of # bytes (goes after SEEKTABLE)\n");
-	fprintf(stderr, "         (0 => no PADDING block, default is -P 0)\n");
-	fprintf(stderr, "  -b # : specify blocksize in samples; default is 1152 for -l 0, else 4608;\n");
-	fprintf(stderr, "         must be 192/576/1152/2304/4608/256/512/1024/2048/4096/8192/16384/32768\n");
-	fprintf(stderr, "         (unless --lax is used)\n");
-	fprintf(stderr, "  -m   : try mid-side coding for each frame (stereo input only)\n");
-	fprintf(stderr, "  -M   : loose mid-side coding for all frames (stereo input only)\n");
-	fprintf(stderr, "  -0 .. -9 : fastest compression .. highest compression, default is -5\n");
-	fprintf(stderr, "             these are synonyms for other options:\n");
-	fprintf(stderr, "  -0   : synonymous with -l 0 -b 1152 -r 2,2\n");
-	fprintf(stderr, "  -1   : synonymous with -l 0 -b 1152 -M -r 2,2\n");
-	fprintf(stderr, "  -2   : synonymous with -l 0 -b 1152 -m -r 3\n");
-	fprintf(stderr, "  -3   : synonymous with -l 6 -b 4608 -r 3,3\n");
-	fprintf(stderr, "  -4   : synonymous with -l 8 -b 4608 -M -r 3,3\n");
-	fprintf(stderr, "  -5   : synonymous with -l 8 -b 4608 -m -r 3,3\n");
-	fprintf(stderr, "  -6   : synonymous with -l 8 -b 4608 -m -r 4\n");
-	fprintf(stderr, "  -7   : synonymous with -l 8 -b 4608 -m -e -r 6\n");
-	fprintf(stderr, "  -8   : synonymous with -l 12 -b 4608 -m -e -r 6\n");
-	fprintf(stderr, "  -9   : synonymous with -l 32 -b 4608 -m -e -E -r 16 -p (very slow!)\n");
-	fprintf(stderr, "  -e   : do exhaustive model search (expensive!)\n");
-	fprintf(stderr, "  -E   : include escape coding in the entropy coder\n");
-	fprintf(stderr, "  -l # : specify max LPC order; 0 => use only fixed predictors\n");
-	fprintf(stderr, "  -p   : do exhaustive search of LP coefficient quantization (expensive!);\n");
-	fprintf(stderr, "         overrides -q, does nothing if using -l 0\n");
-	fprintf(stderr, "  -q # : specify precision in bits of quantized linear-predictor coefficients;\n");
-	fprintf(stderr, "         0 => let encoder decide (min is %u, default is -q 0)\n", FLAC__MIN_QLP_COEFF_PRECISION);
-	fprintf(stderr, "  -r [#,]# : [min,]max residual partition order (# is 0..16; min defaults to 0;\n");
-	fprintf(stderr, "         default is -r 0; above 4 doesn't usually help much)\n");
-	fprintf(stderr, "  -R # : Rice parameter search distance (# is 0..32; above 2 doesn't help much)\n");
-	fprintf(stderr, "  -V   : verify a correct encoding by decoding the output in parallel and\n");
-	fprintf(stderr, "         comparing to the original\n");
-	fprintf(stderr, "  -S-, -m-, -M-, -e-, -E-, -p-, -V-, --delete-input-file-, --lax-, --sector-align-\n");
-	fprintf(stderr, "  can all be used to turn off a particular option\n");
-	fprintf(stderr, "format options:\n");
-	fprintf(stderr, "  -fb | -fl : big-endian | little-endian byte order\n");
-	fprintf(stderr, "  -fc channels\n");
-	fprintf(stderr, "  -fp bits_per_sample\n");
-	fprintf(stderr, "  -fs sample_rate : in Hz\n");
-	fprintf(stderr, "  -fu : unsigned samples (default is signed)\n");
-	fprintf(stderr, "  -fr : force to raw format (even if filename ends in .wav)\n");
-	fprintf(stderr, "  -fw : force to RIFF WAVE\n");
+	fprintf(stderr, "To decode:\n");
+	fprintf(stderr, "  flac -d [infile [...]]\n");
+	fprintf(stderr, "\n");
+	fprintf(stderr, "To test:\n");
+	fprintf(stderr, "  flac -t [infile [...]]\n");
+
+	return message? 1 : 0;
+}
+
+int long_usage(const char *message, ...)
+{
+	FILE *out = (message? stderr : stdout);
+	va_list args;
+
+	if(message) {
+		va_start(args, message);
+
+		(void) vfprintf(stderr, message, args);
+
+		va_end(args);
+
+	}
+	usage_header();
+	fprintf(out, "Usage:\n");
+	fprintf(out, "  flac [options] [infile [...]]\n");
+	fprintf(out, "\n");
+	fprintf(out, "For encoding:\n");
+	fprintf(out, "  the input file(s) may be a PCM RIFF WAVE file or raw samples\n");
+	fprintf(out, "  the output file(s) will be in FLAC format\n");
+	fprintf(out, "For decoding, the reverse is true\n");
+	fprintf(out, "\n");
+	fprintf(out, "A single 'infile' may be - for stdin.  No 'infile' implies stdin.  Use of\n");
+	fprintf(out, "stdin implies -c (write to stdout).  Normally you should use:\n");
+	fprintf(out, "   flac [options] -o outfilename  or  flac -d [options] -o outfilename\n");
+	fprintf(out, "instead of:\n");
+	fprintf(out, "   flac [options] > outfilename   or  flac -d [options] > outfilename\n");
+	fprintf(out, "since the former allows flac to seek backwards to write the STREAMINFO or\n");
+	fprintf(out, "RIFF WAVE header contents when necessary.\n");
+	fprintf(out, "\n");
+	fprintf(out, "flac checks for the presence of a RIFF WAVE header to decide whether or not\n");
+	fprintf(out, "to treat an input file as WAVE format or raw samples.  If any infile is raw\n");
+	fprintf(out, "you must specify the format options {-fb|fl} -fc -fp and -fs, which will\n");
+	fprintf(out, "apply to all raw files.  You can force WAVE files to be treated as a raw files\n");
+	fprintf(out, "using -fr.\n");
+	fprintf(out, "\n");
+	fprintf(out, "generic options:\n");
+	fprintf(out, "  -d : decode (default behavior is encode)\n");
+	fprintf(out, "  -t : test (same as -d except no decoded file is written)\n");
+	fprintf(out, "  -a : analyze (same as -d except an analysis file is written)\n");
+	fprintf(out, "  -c : write output to stdout\n");
+	fprintf(out, "  -s : silent (do not write runtime encode/decode statistics)\n");
+	fprintf(out, "  -o filename : force the output file name (usually flac just changes the\n");
+	fprintf(out, "                extension)\n");
+	fprintf(out, "  --delete-input-file : deletes the input file after a successful encode/decode\n");
+	fprintf(out, "  --skip samples : can be used both for encoding and decoding\n");
+	fprintf(out, "analyze options:\n");
+	fprintf(out, "  --a-rtext : include residual signal in text output\n");
+	fprintf(out, "  --a-rgp : generate gnuplot files of residual distribution of each subframe\n");
+	fprintf(out, "encoding options:\n");
+#ifdef FLAC__HAS_OGG
+	fprintf(out, "  --ogg : output Ogg-FLAC stream instead of native FLAC\n");
+#endif
+	fprintf(out, "  --lax : allow encoder to generate non-Subset files\n");
+	fprintf(out, "  --sector-align : align encoding of multiple files on sector boundaries\n");
+	fprintf(out, "  -S { # | X | #x } : include a point or points in a SEEKTABLE\n");
+	fprintf(out, "       #  : a specific sample number for a seek point\n");
+	fprintf(out, "       X  : a placeholder point (always goes at the end of the SEEKTABLE)\n");
+	fprintf(out, "       #x : # evenly spaced seekpoints, the first being at sample 0\n");
+	fprintf(out, "     You may use many -S options; the resulting SEEKTABLE will be the unique-\n");
+	fprintf(out, "           ified union of all such values.\n");
+	fprintf(out, "     With no -S options, flac defaults to '-S 100x'.  Use -S- for no SEEKTABLE.\n");
+	fprintf(out, "     Note: -S #x will not work if the encoder can't determine the input size\n");
+	fprintf(out, "           before starting.\n");
+	fprintf(out, "     Note: if you use -S # and # is >= samples in the input, there will be\n");
+	fprintf(out, "           either no seek point entered (if the input size is determinable\n");
+	fprintf(out, "           before encoding starts) or a placeholder point (if input size is not\n");
+	fprintf(out, "           determinable)\n");
+	fprintf(out, "  -P # : write a PADDING block of # bytes (goes after SEEKTABLE)\n");
+	fprintf(out, "         (0 => no PADDING block, default is -P 0)\n");
+	fprintf(out, "  -b # : specify blocksize in samples; default is 1152 for -l 0, else 4608;\n");
+	fprintf(out, "         must be 192/576/1152/2304/4608/256/512/1024/2048/4096/8192/16384/32768\n");
+	fprintf(out, "         (unless --lax is used)\n");
+	fprintf(out, "  -m   : try mid-side coding for each frame (stereo input only)\n");
+	fprintf(out, "  -M   : loose mid-side coding for all frames (stereo input only)\n");
+	fprintf(out, "  -0 .. -8 : fastest compression .. highest compression, default is -5\n");
+	fprintf(out, "             these are synonyms for other options:\n");
+	fprintf(out, "  -0   : synonymous with -l 0 -b 1152 -r 2,2\n");
+	fprintf(out, "  -1   : synonymous with -l 0 -b 1152 -M -r 2,2\n");
+	fprintf(out, "  -2   : synonymous with -l 0 -b 1152 -m -r 3\n");
+	fprintf(out, "  -3   : synonymous with -l 6 -b 4608 -r 3,3\n");
+	fprintf(out, "  -4   : synonymous with -l 8 -b 4608 -M -r 3,3\n");
+	fprintf(out, "  -5   : synonymous with -l 8 -b 4608 -m -r 3,3\n");
+	fprintf(out, "  -6   : synonymous with -l 8 -b 4608 -m -r 4\n");
+	fprintf(out, "  -7   : synonymous with -l 8 -b 4608 -m -e -r 6\n");
+	fprintf(out, "  -8   : synonymous with -l 12 -b 4608 -m -e -r 6\n");
+	fprintf(out, "  -e   : do exhaustive model search (expensive!)\n");
+	fprintf(out, "  -E   : include escape coding in the entropy coder\n");
+	fprintf(out, "  -l # : specify max LPC order; 0 => use only fixed predictors\n");
+	fprintf(out, "  -p   : do exhaustive search of LP coefficient quantization (expensive!);\n");
+	fprintf(out, "         overrides -q, does nothing if using -l 0\n");
+	fprintf(out, "  -q # : specify precision in bits of quantized linear-predictor coefficients;\n");
+	fprintf(out, "         0 => let encoder decide (min is %u, default is -q 0)\n", FLAC__MIN_QLP_COEFF_PRECISION);
+	fprintf(out, "  -r [#,]# : [min,]max residual partition order (# is 0..16; min defaults to 0;\n");
+	fprintf(out, "         default is -r 0; above 4 doesn't usually help much)\n");
+	fprintf(out, "  -R # : Rice parameter search distance (# is 0..32; above 2 doesn't help much)\n");
+	fprintf(out, "  -V   : verify a correct encoding by decoding the output in parallel and\n");
+	fprintf(out, "         comparing to the original\n");
+	fprintf(out, "  -S-, -m-, -M-, -e-, -E-, -p-, -V-, --delete-input-file-,%s --lax-, --sector-align-\n",
+#ifdef FLAC__HAS_OGG
+		" --ogg-,"
+#else
+		""
+#endif
+	);
+	fprintf(out, "  can all be used to turn off a particular option\n");
+	fprintf(out, "format options:\n");
+	fprintf(out, "  -fb | -fl : big-endian | little-endian byte order\n");
+	fprintf(out, "  -fc channels\n");
+	fprintf(out, "  -fp bits_per_sample\n");
+	fprintf(out, "  -fs sample_rate : in Hz\n");
+	fprintf(out, "  -fu : unsigned samples (default is signed)\n");
+	fprintf(out, "  -fr : force input to be treated as raw samples\n");
 
 	return message? 1 : 0;
 }
@@ -559,8 +614,10 @@ int encode_file(const char *infilename, const char *forced_outfilename, FLAC__bo
 	char *p;
 	FLAC__byte lookahead[12];
 	unsigned lookahead_length = 0;
+	FLAC__bool treat_as_wave = false;
 	int retval;
 	long infilesize;
+	encode_options_t common_options;
 
 	if(0 == strcmp(infilename, "-")) {
 		infilesize = -1;
@@ -574,95 +631,44 @@ int encode_file(const char *infilename, const char *forced_outfilename, FLAC__bo
 		}
 	}
 
-	if(sector_align && !format_is_wave && infilesize < 0) {
+	if(!force_raw_format) {
+		/* first set format based on name */
+		if(0 == strcasecmp(infilename+(strlen(infilename)-4), ".wav"))
+			treat_as_wave = true;
+		else
+			treat_as_wave = false;
+
+		/* attempt to guess the file type based on the first 12 bytes */
+		if((lookahead_length = fread(lookahead, 1, 12, encode_infile)) < 12) {
+			if(treat_as_wave)
+				fprintf(stderr, "WARNING: %s is not a WAVE file, treating as a raw file\n", infilename);
+			treat_as_wave = false;
+		}
+		else {
+			if(strncmp(lookahead, "RIFF", 4) || strncmp(lookahead+8, "WAVE", 4)) {
+				if(treat_as_wave)
+					fprintf(stderr, "WARNING: %s is not a WAVE file, treating as a raw file\n", infilename);
+				treat_as_wave = false;
+			}
+			else
+				treat_as_wave = true;
+		}
+	}
+
+	if(sector_align && !treat_as_wave && infilesize < 0) {
 		fprintf(stderr, "ERROR: can't --sector-align when the input size is unknown\n");
 		return 1;
 	}
 
-	if(format_is_wave < 0) {
-		/* first set format based on name */
-		if(0 == strcasecmp(infilename+(strlen(infilename)-4), ".wav"))
-			format_is_wave = true;
-		else
-			format_is_wave = false;
-
-		/* attempt to guess the file type based on the first 12 bytes */
-		if((lookahead_length = fread(lookahead, 1, 12, encode_infile)) < 12) {
-			if(format_is_wave)
-				fprintf(stderr, "WARNING: %s is not a WAVE file, treating as a raw file\n", infilename);
-			format_is_wave = false;
-		}
-		else {
-			if(strncmp(lookahead, "RIFF", 4) || strncmp(lookahead+8, "WAVE", 4)) {
-				if(format_is_wave)
-					fprintf(stderr, "WARNING: %s is not a WAVE file, treating as a raw file\n", infilename);
-				format_is_wave = false;
-			}
-			else
-				format_is_wave = true;
-		}
-	}
-
-	if(!format_is_wave) {
+	if(!treat_as_wave) {
 		if(format_is_big_endian < 0 || format_channels < 0 || format_bps < 0 || format_sample_rate < 0)
-			return usage("ERROR: for encoding a raw file you must specify { -fb or -fl }, -fc, -fp, and -fs\n");
+			return long_usage("ERROR: for encoding a raw file you must specify { -fb or -fl }, -fc, -fp, and -fs\n");
 	}
 
 	if(encode_infile == stdin || force_to_stdout)
 		strcpy(outfilename, "-");
 	else {
-		strcpy(outfilename, output_prefix? output_prefix : "");
-		strcat(outfilename, infilename);
-		if(0 == (p = strrchr(outfilename, '.')))
-			strcat(outfilename, ".flac");
-		else {
-			if(0 == strcmp(p, ".flac"))
-				strcpy(p, "_new.flac");
-			else
-				strcpy(p, ".flac");
-		}
-	}
-	if(0 == forced_outfilename)
-		forced_outfilename = outfilename;
-	if(0 != cmdline_forced_outfilename)
-		forced_outfilename = cmdline_forced_outfilename;
-
-	if(format_is_wave)
-		retval = flac__encode_wav(encode_infile, infilesize, infilename, forced_outfilename, lookahead, lookahead_length, align_reservoir, &align_reservoir_samples, sector_align, is_last_file, verbose, skip, verify, lax, do_mid_side, loose_mid_side, do_exhaustive_model_search, do_escape_coding, do_qlp_coeff_prec_search, min_residual_partition_order, max_residual_partition_order, rice_parameter_search_dist, max_lpc_order, (unsigned)blocksize, qlp_coeff_precision, padding, requested_seek_points, num_requested_seek_points);
-	else
-		retval = flac__encode_raw(encode_infile, infilesize, infilename, forced_outfilename, lookahead, lookahead_length, is_last_file, verbose, skip, verify, lax, do_mid_side, loose_mid_side, do_exhaustive_model_search, do_escape_coding, do_qlp_coeff_prec_search, min_residual_partition_order, max_residual_partition_order, rice_parameter_search_dist, max_lpc_order, (unsigned)blocksize, qlp_coeff_precision, padding, requested_seek_points, num_requested_seek_points, format_is_big_endian, format_is_unsigned_samples, format_channels, format_bps, format_sample_rate);
-
-	if(retval == 0 && strcmp(infilename, "-")) {
-		if(strcmp(forced_outfilename, "-"))
-			flac__file_copy_metadata(infilename, forced_outfilename);
-		if(delete_input)
-			unlink(infilename);
-	}
-
-	return 0;
-}
-
-int decode_file(const char *infilename, const char *forced_outfilename)
-{
-	static const char *suffixes[] = { ".wav", ".raw", ".ana" };
-	char outfilename[4096]; /* @@@ bad MAGIC NUMBER */
-	char *p;
-	int retval;
-
-	if(!test_only && !analyze) {
-		if(format_is_wave < 0) {
-			format_is_wave = true;
-		}
-		if(!format_is_wave) {
-			if(format_is_big_endian < 0)
-				return usage("ERROR: for decoding to a raw file you must specify -fb or -fl\n");
-		}
-	}
-
-	if(0 == strcmp(infilename, "-") || force_to_stdout)
-		strcpy(outfilename, "-");
-	else {
-		const char *suffix = suffixes[analyze? 2 : format_is_wave? 0 : 1];
+		const char *suffix = (use_ogg? ogg_suffix : flac_suffix);
 		strcpy(outfilename, output_prefix? output_prefix : "");
 		strcat(outfilename, infilename);
 		if(0 == (p = strrchr(outfilename, '.')))
@@ -681,10 +687,134 @@ int decode_file(const char *infilename, const char *forced_outfilename)
 	if(0 != cmdline_forced_outfilename)
 		forced_outfilename = cmdline_forced_outfilename;
 
-	if(format_is_wave)
-		retval = flac__decode_wav(infilename, test_only? 0 : forced_outfilename, analyze, aopts, verbose, skip);
+	common_options.verbose = verbose;
+	common_options.skip = skip;
+	common_options.verify = verify;
+#ifdef FLAC__HAS_OGG
+	common_options.use_ogg = use_ogg;
+#endif
+	common_options.lax = lax;
+	common_options.do_mid_side = do_mid_side;
+	common_options.loose_mid_side = loose_mid_side;
+	common_options.do_exhaustive_model_search = do_exhaustive_model_search;
+	common_options.do_escape_coding = do_escape_coding;
+	common_options.do_qlp_coeff_prec_search = do_qlp_coeff_prec_search;
+	common_options.min_residual_partition_order = min_residual_partition_order;
+	common_options.max_residual_partition_order = max_residual_partition_order;
+	common_options.rice_parameter_search_dist = rice_parameter_search_dist;
+	common_options.max_lpc_order = max_lpc_order;
+	common_options.blocksize = (unsigned)blocksize;
+	common_options.qlp_coeff_precision = qlp_coeff_precision;
+	common_options.padding = padding;
+	common_options.requested_seek_points = requested_seek_points;
+	common_options.num_requested_seek_points = num_requested_seek_points;
+
+	if(treat_as_wave) {
+		wav_encode_options_t options;
+
+		options.common = common_options;
+		options.is_last_file = is_last_file;
+		options.align_reservoir = align_reservoir;
+		options.align_reservoir_samples = &align_reservoir_samples;
+		options.sector_align = sector_align;
+
+		retval = flac__encode_wav(encode_infile, infilesize, infilename, forced_outfilename, lookahead, lookahead_length, options);
+	}
+	else {
+		raw_encode_options_t options;
+
+		options.common = common_options;
+		options.is_big_endian = format_is_big_endian;
+		options.is_unsigned_samples = format_is_unsigned_samples;
+		options.channels = format_channels;
+		options.bps = format_bps;
+		options.sample_rate = format_sample_rate;
+
+		retval = flac__encode_raw(encode_infile, infilesize, infilename, forced_outfilename, lookahead, lookahead_length, options);
+	}
+
+	if(retval == 0 && strcmp(infilename, "-")) {
+		if(strcmp(forced_outfilename, "-"))
+			flac__file_copy_metadata(infilename, forced_outfilename);
+		if(delete_input)
+			unlink(infilename);
+	}
+
+	return 0;
+}
+
+int decode_file(const char *infilename, const char *forced_outfilename)
+{
+	static const char *suffixes[] = { ".wav", ".raw", ".ana" };
+	char outfilename[4096]; /* @@@ bad MAGIC NUMBER */
+	char *p;
+	int retval;
+	FLAC__bool treat_as_ogg = false;
+	decode_options_t common_options;
+
+	if(!test_only && !analyze) {
+		if(force_raw_format && format_is_big_endian < 0)
+			return long_usage("ERROR: for decoding to a raw file you must specify -fb or -fl\n");
+	}
+
+	if(0 == strcmp(infilename, "-") || force_to_stdout)
+		strcpy(outfilename, "-");
+	else {
+		const char *suffix = suffixes[analyze? 2 : force_raw_format? 1 : 0];
+		strcpy(outfilename, output_prefix? output_prefix : "");
+		strcat(outfilename, infilename);
+		if(0 == (p = strrchr(outfilename, '.')))
+			strcat(outfilename, suffix);
+		else {
+			if(0 == strcmp(p, suffix)) {
+				strcpy(p, "_new");
+				strcat(p, suffix);
+			}
+			else
+				strcpy(p, suffix);
+		}
+	}
+	if(0 == forced_outfilename)
+		forced_outfilename = outfilename;
+	if(0 != cmdline_forced_outfilename)
+		forced_outfilename = cmdline_forced_outfilename;
+
+	if(use_ogg)
+		treat_as_ogg = true;
+	else if(0 == strcasecmp(infilename+(strlen(infilename)-4), ".ogg"))
+		treat_as_ogg = true;
 	else
-		retval = flac__decode_raw(infilename, test_only? 0 : forced_outfilename, analyze, aopts, verbose, skip, format_is_big_endian, format_is_unsigned_samples);
+		treat_as_ogg = false;
+
+#ifndef FLAC__HAS_OGG
+	if(treat_as_ogg) {
+		fprintf(stderr, "%s: Ogg support has not been built into this copy of flac\n", infilename);
+		return 1;
+	}
+#endif
+
+	common_options.verbose = verbose;
+#ifdef FLAC__HAS_OGG
+	common_options.is_ogg = treat_as_ogg;
+#endif
+	common_options.skip = skip;
+
+	if(!force_raw_format) {
+		wav_decode_options_t options;
+
+		options.common = common_options;
+
+		retval = flac__decode_wav(infilename, test_only? 0 : forced_outfilename, analyze, aopts, options);
+	}
+	else {
+		raw_decode_options_t options;
+
+		options.common = common_options;
+		options.is_big_endian = format_is_big_endian;
+		options.is_unsigned_samples = format_is_unsigned_samples;
+
+		retval = flac__decode_raw(infilename, test_only? 0 : forced_outfilename, analyze, aopts, options);
+	}
 
 	if(retval == 0 && strcmp(infilename, "-")) {
 		if(strcmp(forced_outfilename, "-"))
